import {
  ClassType,
} from '@overflow/commons/core/type';

import {
  InstanceFactory,
} from '@overflow/commons/di/factory';

import { ApplicationContextIDType } from '@overflow/commons/application/type';

import { MessageSource } from './message_source';
import { InstanceDefinitionReader } from './instance_definition_reader';

export class ApplicationContext {
  public static readonly MESSAGE_SOURCE_INSTANCE_NAME: string = 'messageSource';

  private _id: ApplicationContextIDType;
  private _parent: ApplicationContext;
  private _messageSource: MessageSource;
  private _instanceFactory: InstanceFactory;
  private _instanceDefinitionReader: InstanceDefinitionReader;

  public constructor(parent?: ApplicationContext) {
    this.parent = parent;
    this._instanceFactory = new InstanceFactory();
    this._instanceDefinitionReader = new InstanceDefinitionReader(this._instanceFactory.instanceDefinitionRegistry);
  }


  // Context

  public get id(): ApplicationContextIDType {
    return this.id;
  }
  public set id(id: ApplicationContextIDType) {
    this.id = id;
  }

  public get applicationName(): string {
    return undefined;
  }

  public get parent(): ApplicationContext {
    return this._parent;
  }
  public set parent(parent: ApplicationContext) {
    this._parent = parent;
  }

  public get instanceFactory(): InstanceFactory {
    return this._instanceFactory;
  }

  public get instanceDefinitionReader(): InstanceDefinitionReader {
    return this._instanceDefinitionReader;
  }

  // Configuration

  // MessageSource
  public get messageSource(): MessageSource {
    return this._messageSource;
  }

  // Lifecycle

  public start(): void {
    //
  }
  public stop(): void {
    //
  }
  public isRunning(): boolean {
    return false;
  }
}
