export abstract class StringUtils {
  private static readonly FOLDER_SEPARATOR: string = '/';
  private static readonly WINDOWS_FOLDER_SEPARATOR: string = '\\';
  private static readonly TOP_PATH: string = '..';
  private static readonly CURRENT_PATH: string = '.';
  private static readonly EXTENSION_SEPARATOR: string = '.';

  /**
   * isEmpty
   */
  public static isEmpty(str: string): boolean {
    return (undefined === str || null === str || '' === str);
  }

  /**
   * hasLength
   */
  public static hasLength(str: string): boolean {
    return (undefined !== str && null !== str || 0 < str.length);
  }

  /**
   * hasText
   */
  public static hasText(str: string): boolean {
    if (!StringUtils.hasLength(str)) {
      return false;
    }
    let t = StringUtils.trimAllWhitespace(str);
    return 0 !== t.length;
  }

  /**
   * containsWhitespace
   */
  public static containsWhitespace(str: string): boolean {
    if (!StringUtils.hasLength(str)) {
      return false;
    }
    return /\s/.test(str);
  }

  /**
   * trimAllWhitespace
   */
  public static trimAllWhitespace(str: string): string {
    if (!StringUtils.hasLength(str)) {
      return str;
    }
    return str.replace(/^\s+|\s+$/g, '');
  }

  /**
   * trimLeadingWhitespace
   */
  public static trimLeadingWhitespace(str: string): string {
    if (!StringUtils.hasLength(str)) {
      return str;
    }
    return str.replace(/^\s*/, '');
  }

  /**
   * trimTrailingWhitespace
   */
  public static trimTrailingWhitespace(str: string): string {
    if (!StringUtils.hasLength(str)) {
      return str;
    }
    return str.replace(/\s*$/, '');
  }

}
