import {
  Annotation,
  Decorator,
} from '@overflow/commons/core/reflect';

import {
  PropertyKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';

import {
  InstanceNameType,
} from '@overflow/commons/di/type';

export interface QualifierAnnotationAttributes {
  name?: InstanceNameType;
}

export class QualifierAnnotation extends Annotation {
  public readonly attributes: QualifierAnnotationAttributes = {
    name: '',
  };

  public constructor(name: InstanceNameType | QualifierAnnotationAttributes) {
    super();
    if (undefined === name) {
      return;
    }

    if (TypeUtil.isString(name)) {
      this.attributes.name = <InstanceNameType>name;
    } else {
      this.copyAttributes(this.attributes, <QualifierAnnotationAttributes>name);
    }
  }

  public classDecorator<TFunction extends Function>(target: TFunction): TFunction | void {
    console.log('Qualifier');
  }

  public propertyDecorator(target: Object, propertyKey: PropertyKeyType): void {
    console.log('Qualifier');
  }

  public methodDecorator<T>(target: Object, propertyKey: PropertyKeyType,
                            descriptor: TypedPropertyDescriptor<T>): TypedPropertyDescriptor<T> | void {
    console.log('Qualifier');
  }

  public parameterDecorator(target: Object, propertyKey: PropertyKeyType, parameterIndex: number): void {
    console.log('Qualifier');
  }
}

export const Qualifier = Decorator.create(QualifierAnnotation);
